
#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"

#include "uicr_config.h"
#include <stdbool.h>
#include "app_util.h"
#include "stdio.h"
#include "string.h"
#include "nrf_fstorage.h"
#include "nrf_fstorage_sd.h"
#include "nrf_delay.h"
#include "nrf_nvic.h"

#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"


static uint8_t DEVICE_NAME[16];
ret_code_t rc_ret;

/*------------------------------ < UICR Process Start > ------------------------------*/
#define FLASH_ADDR  		0x10001000					/* [NRF_UICR] register Address setting */
#define FLASH_START_ADRS	0x10001080					/* [NRF_UICR->CUSTOMER] register start Address setting   */
#define FLASH_END_ADRS		0x100010FF					/* [NRF_UICR->CUSTOMER] register end Address setting   */

/*----------------------------------------*/
/*-< UICR Event handler (TEST)          >-*/
/*----------------------------------------*/
void fs_evt_handler(nrf_fstorage_evt_t * p_evt);


void fs_evt_handler(nrf_fstorage_evt_t * p_evt)	
{	
    rc_ret = 1;
    if (p_evt->result != NRF_SUCCESS)
    {
        return;
    }

    switch (p_evt->id)
    {
		case NRF_FSTORAGE_EVT_READ_RESULT: 
        {
			printf("wrote %d bytes at address 0x%x.\r\n",p_evt->len, p_evt->addr);
//			uint32_t err_code = sd_nvic_SystemReset();
        } break; 

        case NRF_FSTORAGE_EVT_WRITE_RESULT:
        {
			printf("wrote %d bytes at address 0x%x.\r\n",p_evt->len, p_evt->addr);
//			uint32_t err_code = sd_nvic_SystemReset();
        } break; 

        case NRF_FSTORAGE_EVT_ERASE_RESULT:
        {
			printf("erased %d page from address 0x%x.\r\n",	p_evt->len, p_evt->addr);
//			uint32_t err_code = sd_nvic_SystemReset();
        } break;

        default:
            break;
    }		
}

NRF_FSTORAGE_DEF(nrf_fstorage_t my_instance) =
{
	.evt_handler    = fs_evt_handler,
	.start_addr     = FLASH_START_ADRS,
	.end_addr       = FLASH_END_ADRS,
};

/*----------------------------------------*/
/*-< UICR Initialization (TEST)         >-*/
/*----------------------------------------*/
void UICR_Initial(void)
{
	nrf_fstorage_init(
		&my_instance,				/* You fstorage instance, previously defined. */
		&nrf_fstorage_sd,			/* Name of the backend. */
		NULL						/* Optional parameter, backend-dependant. */
	);
}

/*----------------------------------------*/
/*-< UICR Writing data (TEST)           >-*/
/*----------------------------------------*/
void UICR_Write_data( uint8_t *device_name )
{
	rc_ret = nrf_fstorage_write(
		&my_instance,				/* The instance to use. */
		FLASH_START_ADRS,			/* The address in flash where to store the data. */
		&device_name,				/* A pointer to the data. */
		sizeof(device_name),		/* Lenght of the data, in bytes. */
		NULL						/* Optional parameter, backend-dependent. */
	);
	if (rc_ret == NRF_SUCCESS)
	{
		/* The operation was accepted. */
	}else{
		/* Handle error.*/
	}
}

/*----------------------------------------*/
/*-< UICR Writing data (TEST)           >-*/
/*----------------------------------------*/
void UICR_Read_data( uint8_t *device_name )
{
	rc_ret = nrf_fstorage_read(
		&my_instance,				/* The instance to use. */
		FLASH_START_ADRS,			/* The address in flash where to read data from. */
		&device_name,				/* A buffer to copy the data into. */
		sizeof(device_name)		/* Lenght of the data, in bytes. */
	);
	if (rc_ret == NRF_SUCCESS)
	{
		/* The operation was accepted. */
	}else{
		/* Handle error.*/
	}
}
/*----------------------------------------*/
/*-< UICR Erasing flash pages (TEST)    >-*/
/*----------------------------------------*/
void UICR_Erase_flash_page(uint32_t len )
{
	rc_ret = nrf_fstorage_erase(
		&my_instance,				/* The instance to use. */
		FLASH_START,			/* The address of the flash pages to erase. */
		len,						/* The number of pages to erase. */
		NULL						/* Optional parameter, backend-dependent. */
	);
	if (rc_ret == NRF_SUCCESS)
	{
		/* The operation was accepted.
			Upon completion, the NRF_FSTORAGE_ERASE_RESULT event
			is sent to the callback function registered by the instance. */
	}else{
		/* Handle error.*/
	}
}

/**@brief Function for handling the idle state (main loop). */
static void idle_state_handle(void)
{
    if (NRF_LOG_PROCESS() == false)
    {
        nrf_pwr_mgmt_run();
    }
}

/**@brief Application main function. */
int main(void)
{
    bool erase_bonds;
	uint8_t DEVICE_NAME_INI1[] = "1234567890";
	uint8_t DEVICE_NAME_INI2[] = "0987654321";

    UICR_Initial();

	memcpy( DEVICE_NAME, DEVICE_NAME_INI1, sizeof(DEVICE_NAME_INI1));			/* dumy data setting */
    UICR_Read_data(DEVICE_NAME);												/* Reading data from flash ROM */

    UICR_Erase_flash_page (sizeof(DEVICE_NAME_INI2));							/* Erase flash ROM */

	memcpy( DEVICE_NAME, DEVICE_NAME_INI2, sizeof(DEVICE_NAME_INI2));			/* Write data setting */
    UICR_Write_data(DEVICE_NAME);												/* Write data to flash ROM */
 
    // Enter main loop.
    for (;;)
    {
        idle_state_handle();
    }
}


/**
 * @}
 */
